﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Globalization;
using System.Collections.ObjectModel;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Diagnostics;
using System.Windows.Threading;
using TestMapSuiteWPF.Components;



namespace TestShapeFile
{
   
   
    /// <summary>
    /// Interaction logic for MapControl.xaml
    /// </summary>
    public partial class MapControl : UserControl
    {


        Proj4Projection m_Projection;
        LayerOverlay worldOverlay;
        string CacheDirectory;
        ShapeFileFeatureLayer worldLayer;
        public MapControl()
        {
            InitializeComponent();



            m_Projection = new Proj4Projection();
            m_Projection.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            m_Projection.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
            m_Projection.Open();

            CacheDirectory = AppDomain.CurrentDomain.BaseDirectory + @"Resources\Tiles\";
            wpfMap1.MapUnit = GeographyUnit.Meter;

            worldOverlay = new LayerOverlay();
            worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            wpfMap1.Overlays.Add("test", worldOverlay);

            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();
            bitmapTileCache.ClearCache();
            bitmapTileCache.CacheDirectory = CacheDirectory;
            bitmapTileCache.CacheId = "Mercator";
            bitmapTileCache.TileAccessMode = TileAccessMode.ReadAdd;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;

            bitmapTileCache.TileMatrix.BoundingBoxUnit = wpfMap1.MapUnit;

            // the wrapping date line has some limits that the max bounding box's width and height need to be n times of the real world.
            // the default max extent doesn't follow this rule. We need change it manually here as well as the method we generate the cache.
            bitmapTileCache.TileMatrix.BoundingBox = GenerateTiles.MaxExtentForWrappingDateline;

            worldOverlay.TileCache = bitmapTileCache;

            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle = LineStyles.DegreeLine1;
            graticuleAdornmentLayer.WrappingMode = WrappingMode.WrapDateline;
            graticuleAdornmentLayer.WrappingExtent = new RectangleShape(-20037508, 18464390, 20037508, -20037658);


            wpfMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

         





        }




        private void wpfMap1_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {



                bool success = LoadWorldBackground();
                if (!success) return;




                SetCustomZoomLevels();

                ToSphericalMercator();

                worldLayer.Open();
                GenerateTiles.Generate(wpfMap1.ZoomLevelSet, worldLayer, wpfMap1.MapUnit, 256, 256, CacheDirectory, "Mercator");



                PointShape point = new PointShape(1629717.38343204, -3193612.19591537);

                InMemoryFeatureLayer layer1 = new InMemoryFeatureLayer();
                layer1.InternalFeatures.Add(new Feature(point));

                LayerOverlay overlay1 = new LayerOverlay();
                overlay1.Layers.Add(layer1);
                layer1.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                layer1.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
                overlay1.WrappingMode = WrappingMode.WrapDateline;
                wpfMap1.Overlays.Add(overlay1);





                wpfMap1.Refresh();

                wpfMap1.ExtentOverlay.MapMouseClick += new EventHandler<MapMouseClickInteractiveOverlayEventArgs>(ExtentOverlay_MapMouseClick);





            }
            catch (Exception ex)
            {
                string s = ex.Message;
                throw;
            }


        }
        bool isPopupOpen = false;
        void ExtentOverlay_MapMouseClick(object sender, MapMouseClickInteractiveOverlayEventArgs e)
        {
            if (!isPopupOpen)
            {
                Popup popup = new Popup(e.InteractionArguments.WorldX, e.InteractionArguments.WorldY);

                TextBlock txt = new TextBlock();
                txt.Text = "test";
                popup.Content = txt;

                PopupOverlay popupOverlay = new PopupOverlay();
                popupOverlay.Name = "popup";
                popupOverlay.Popups.Add(popup);
                wpfMap1.Overlays.Add("popup", popupOverlay);

                wpfMap1.Refresh(popupOverlay);
                isPopupOpen = true;
            }
            else
            {
                wpfMap1.Overlays.Remove("popup");
                isPopupOpen = false;
            }
        }

        private void SetCustomZoomLevels()
        {
            wpfMap1.ZoomLevelSet = new GoogleMapZoomLevelSet();

            wpfMap1.CurrentExtent = new RectangleShape(-15000000, 15000000, 15000000, -15000000);
        }


        private void ToSphericalMercator()
        {
            if (wpfMap1.Overlays.Count > 0)
            {



                foreach (Overlay overlay in wpfMap1.Overlays)
                {
                    if (overlay is LayerOverlay)
                    {
                        LayerOverlay lOverlay = (LayerOverlay)overlay;
                        foreach (Layer layer in lOverlay.Layers)
                        {
                          
                            if (layer is FeatureLayer)
                            {
                                FeatureLayer featureLayer = ((FeatureLayer)layer);
                                if (featureLayer.FeatureSource.Projection == null)
                                {
                                    featureLayer.FeatureSource.Projection = m_Projection;

                                    worldLayer.Open();
                                    featureLayer.WrappingMode = WrappingMode.WrapDateline;
                                    featureLayer.WrappingExtent = worldLayer.GetBoundingBox();

                                    worldLayer.Close();
                                }
                            }


                        }
                    }
                }

                foreach (AdornmentLayer layer in wpfMap1.AdornmentOverlay.Layers)
                {

                    if (layer is GraticuleAdornmentLayer)
                    {
                        GraticuleAdornmentLayer graticuleAdornmentLayer = ((GraticuleAdornmentLayer)layer);
                        if (graticuleAdornmentLayer.Projection == null)
                        {
                            graticuleAdornmentLayer.Projection = m_Projection;
                        }


                    }
                }


            }
        }

        public bool LoadWorldBackground()
        {
            string sMapShapeFileName = AppDomain.CurrentDomain.BaseDirectory + @"Resources\map\Countries02.shp";
            string sMapShapeIndexFileName = AppDomain.CurrentDomain.BaseDirectory + @"Resources\map\Countries02.idx";

            if (File.Exists(sMapShapeFileName))
            {
                // ShapeFileFeatureLayer.BuildIndexFile(sMapShapeFileName, BuildIndexMode.Rebuild);
                ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(sMapShapeFileName, sMapShapeIndexFileName);
            
                layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County1;
                layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.InternationalBorder2;
                layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level10;

              
             
                worldLayer = layer;



                worldOverlay.Layers.Add("worldLayer", layer);

            


                return true;
            }
            else
            {
                Debug.Print("Failed to find background shape file ({0}) - using default", sMapShapeFileName);
            }
            return false;
        }

      
       
    }
}
